#include "DictWrapper.h"

#include <QDebug>

DictWrapper::DictWrapper(void)
{
	qDebug() << "DictWrapper::DictWrapper";

	gcide = new Gcide();
}


void DictWrapper::setCompleterWords(QStringList *words)
{
	qDebug() << "DictWrapper::setCompleterWords";

	gcide->setCompleterWords(words);
}


QString DictWrapper::getStdoutFromCommand (const QString cmd)
{
	qDebug() << "DictWrapper::getStdoutFromCommand";

	QString out;

	const int MAX_BUFFER = 256;
	char buffer[MAX_BUFFER];

	FILE *stream = popen(cmd.toStdString().c_str(), "r");

	while (fgets(buffer, MAX_BUFFER, stream) != NULL) {
		out.append(buffer);
	}
	if (ferror(stream)) {
		return "Error accured while executing the command";
	}

	pclose(stream);
	return out;
}


void DictWrapper::parseDefault (QStringList::iterator it)
{
	qDebug() << "DictWrapper::parseDefault";

	QRegExp rx_from("^From");
	while (it!=lines.end() && rx_from.indexIn(*it, 0) == -1) {
		(*it).append("<br>");
		it++;
	}
}


void DictWrapper::formatAsHtml (const QString title)
{
	qDebug() << "DictWrapper::formatAsHtml";

	QRegExp rx_from("^From.*\\[(.*)\\]:$");

	QStringList::iterator it;
	for (it = lines.begin(); it!=lines.end(); it++){

		// is this line a dict header
		if ( rx_from.indexIn(*it, 0) != -1) {
			(*it).prepend("<b>");
			(*it).append("</b>");

			(*it).prepend("<p>");

			// different parser method for different dictionaries
			if ( rx_from.cap(1) == "gcide" ) {
				gcide->parser(++it,lines.last(), lines.end());
			} else {
				parseDefault(++it);
			}

			(*it).append("</p>");
		}
	}

	// the necessary html tags
	lines.prepend("<html>\n<head>\n<title>"+title+
			"</title>\n</head>\n<body>\n");
	lines.prepend("<h3>"+title+"</h3>");
	lines.append("</body>\n</html>\n");
}


bool DictWrapper::definitionsFound(void)
{
	qDebug() << "DictWrapper::definitionsFound";

	QRegExp rx_noDef("^No (matches|definitions) found for");

	QStringList::iterator it = lines.begin();
	return (rx_noDef.indexIn(*it, 0) != -1) ? false : true;
}


void DictWrapper::createLinksPage(const QString string_to_translate)
{
	lines.clear();

	lines.append("<html>\n<head>\n<title>");
	lines.append("No definitions found");
	lines.append("</title>\n</head>\n<body>\n");
	lines.append("<h3>No definitions found for ");
	lines.append(string_to_translate);

	QString chunk(string_to_translate);
	chunk.replace(QRegExp("\\s+"), "+");

	lines.append("</h3>Try the following online search engines:<p><p>");

	lines.append("Google.com <a href=\"http://www.google.com" \
		"/search?q="+chunk+"\">link</a><p>");
	lines.append("WordReference.com <a href=\"http://www." \
		"wordreference.com/definition/"+chunk+"\">link</a><p>");
	lines.append("Wiktionary.org <a href=\"http://en.wiktionary.org" \
		"/wiki/Special:Search?search="+chunk+"\">link</a><p>");
	lines.append("UrbanDictionary.org <a href=\"http://www." \
		"urbandictionary.com/define.php?term="+chunk+"\">link</a><p>");

	lines.append("</body>\n</html>\n");
}


QString DictWrapper::translateToHtmlString (const QString string_to_translate,
					    const int strategy)
{
	qDebug() << "DictWrapper::translateToHtmlString";

	QString command = "dict -d gcide -s " +
			getSearchStrategies().at(strategy) +
			" \""+ string_to_translate + "\"  2>&1";

	lines = getStdoutFromCommand(command).split('\n');
	if (definitionsFound()) {
		formatAsHtml(string_to_translate);
	} else {
		createLinksPage(string_to_translate);
	}

	return lines.join("\n");
}


QStringList DictWrapper::getSearchStrategies(void)
{
	qDebug() << "DictWrapper::getSearchStrategies";

	QStringList strategies;
	QStringList tmp = getStdoutFromCommand("dict -S").split('\n');
	QRegExp rx_strat("^\\s(\\S+).+$");
	QStringList::iterator it;

	for (it = tmp.begin(); it!=tmp.end(); it++){
		if ( rx_strat.indexIn(*it, 0) != -1) {
			strategies << rx_strat.cap(1);
		}
	}
	return strategies;
}

