#include "Gcide.h"

#include <QDebug>

void Gcide::setCompleterWords(QStringList *words)
{
	qDebug() << "Gcide::setCompleterWords";

	/// @todo The index file path is distro-dependent.
	QFile file;

	QString gentooPath("/usr/lib/dict/web1913.index");
	QString ubuntuPath("/usr/share/dictd/gcide.index");

	file.setFileName(gentooPath);

	if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
		file.setFileName(ubuntuPath);
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
			return;
		}
	}

	QRegExp gcideExpr("^(\\D[^-].*)\\s\\S+\\s\\S+$");
	QString line;
	QString lastMatch; // no duplicates
	QTextStream in(&file);

	while (!in.atEnd()) {
		line = in.readLine();
		gcideExpr.indexIn(line, 0);
		if (gcideExpr.cap(1).length() > 0) {
			if (gcideExpr.cap(1).compare(lastMatch) != 0) {
				*words << gcideExpr.cap(1);
				lastMatch = gcideExpr.cap(1);
			}
		}
	}
}


bool Gcide::markEnumList (QStringList::iterator it)
{
	qDebug() << "Gcide::markEnumList";

	QRegExp rx_enum("^\\s{5}\\d+\\.(.*)");

	// enumerated list element
	if (rx_enum.indexIn(*it, 0) != -1) {
		// start of the list
		if (in_enumlist == false) {
			*it = "<ol><li>" + rx_enum.cap(1);
			in_enumlist = true;
		} else {
			// end of a sublist
			if (in_sublist) {
				in_sublist = false;
				*it = "</li></ul></li><li>" + rx_enum.cap(1);
			} else {
				*it = "</li><li>" + rx_enum.cap(1);
			}
			return true;
		}
	}
	return false;
}


bool Gcide::markSubList (QStringList::iterator it)
{
	qDebug() << "Gcide::markSubList";

	QRegExp rx_sub("^\\s{8}\\(\\D+\\)\\s(.*)");

	// a sublist entry
	if (rx_sub.indexIn(*it, 0) != -1) {
		if (in_sublist == false) {
			in_sublist = true;
			*it = "<ul><li>" + rx_sub.cap(1);
		} else {
			*it = "</li><li>" + rx_sub.cap(1);
		}
		return true;
	}
	return false;
}


bool Gcide::markList (QStringList::iterator it)
{
	qDebug() << "Gcide::markList";

	QRegExp rx_num("^\\s{5}(\\{.*)");

	// a not numerated list entry
	if (rx_num.indexIn(*it, 0) != -1 && prev_line_was_empty) {
		in_list = true;
		if (in_sublist && in_enumlist) {
			in_sublist = false;
			in_enumlist = false;
			*it = "</li></ul></li></ol>Expressions<ul><li>"
				+ rx_num.cap(1);
		} else if (in_enumlist) {
			in_enumlist = false;
			*it = "</li></ol>Expressions<ul><li>"
				+ rx_num.cap(1);
		} else {
			*it = "</li><li>" + rx_num.cap(1);
		}
		return true;
	}
	return false;
}

bool Gcide::markQuotation (QStringList::iterator it)
{
	qDebug() << "Gcide::markQuotation";

	QRegExp rx_quo2("^\\s{11,}(\\S.*)");

	if ( rx_quo2.indexIn(*it, 0) != -1 && prev_line_was_empty ) {
		if (in_quotation == false ) {
			in_quotation = true;
			*it = "<p><i>" + rx_quo2.cap(1);
		}
		return true;
	}
	return false;
}


bool Gcide::markSource (QStringList::iterator &it)
{
	qDebug() << "Gcide::markSource";

	QRegExp rx_src("^([^-]*)--(.*)");
	QRegExp rx_empty("^\\s*$");
	if ( rx_src.indexIn(*it, 0) != -1 && in_quotation == true ) {
		in_quotation = false;
		*it = rx_src.cap(1) + "</i>--" + rx_src.cap(2);
		while (rx_empty.indexIn(*it,0) == -1) {
			it++;
		}
		it-2;
		(*it).append("<b>");
		return true;
	}
	return false;
}


void Gcide::replaceLinks (QStringList::iterator it)
{
	qDebug() << "Gcide::replaceLinks";

	int pos(0);
	QString tmp;
	QRegExp rx_link("(\\{.*\\})");
	rx_link.setMinimal(true);

	// look for links
	/// @todo links which overlaps to the next lines
	while ((pos = rx_link.indexIn(*it, pos)) != -1) {

		tmp = rx_link.cap(1);
		(*it).replace(pos,tmp.length(),
			("<a href=\""+tmp.mid(1,tmp.length()-2)+
			"\">"+tmp.mid(1,tmp.length()-2)+"</a>"));

		pos += rx_link.matchedLength();

	}
}


void Gcide::replaceNote (QStringList::iterator it)
{
	qDebug() << "Gcide::replaceNote";

	QRegExp rx_note("^(\\s{5}Note:)");

	if (rx_note.indexIn(*it, 0) != -1) {
		(*it).replace(0,rx_note.cap(1).length(),"<p><b>Note:</b>");
	}
}


void Gcide::replaceDb (QStringList::iterator it)
{
	qDebug() << "Gcide::replaceDb";

	int pos(0);
	QRegExp rx_source("\\s{8}(\\[.*\\])");

	if ((pos = rx_source.indexIn(*it, 0)) != -1) {

		(*it).replace(pos,rx_source.cap(1).length()+8,
				"<i>"+rx_source.cap(1)+"</i>");
	}
}


void Gcide::parser (	QStringList::iterator it,
				QString last,
				QStringList::iterator end)
{
	qDebug() << "Gcide::parseGcide";

	in_enumlist 		= false;
	in_sublist		= false;
	in_list			= false;
	in_quotation		= false;
	in_source		= false;
	prev_line_was_empty	= false;

	QRegExp rx_from("^From");
	QRegExp rx_empty("^\\s*$");

	while (it!=end && rx_from.indexIn(*it, 0) == -1) {

		markEnumList(it);
		markSubList(it);
		markList(it);
		markQuotation(it);
 		markSource(it);
			

		replaceLinks(it);
		replaceNote(it);
		replaceDb(it);

		if (*it == last) {
			if (in_sublist) (*it).append("</li></ul>");
			if (in_enumlist) (*it).append("</li></ol>");
			if (in_list) (*it).append("</li></ul>");
		}

		prev_line_was_empty = rx_empty.indexIn(*it,0) != -1;

		it++;
	}
}
