#include "MainWidget.h"

#include <QDebug>

MainWidget::MainWidget(QWidget *parent) : QWidget(parent)
{
	qDebug() << "MainWidget::MainWidget";

	mainLayout = new QGridLayout();

	entryComboBox = new QComboBox();
	entryComboBox->setEditable(true);

	dictWrapper = new DictWrapper();

	completerWords = new QStringList();
	dictWrapper->setCompleterWords(completerWords);

	completer = new QCompleter(*completerWords, entryComboBox);
	completer->setCaseSensitivity(Qt::CaseInsensitive);
	entryComboBox->setCompleter(completer);

	// set the entry combobox to the content text of the clipbord
	entryComboBox->setEditText(QApplication::clipboard()->text());

	strategiesLabel = new QLabel("strategy");

	searchStrategyComboBox = new QComboBox();
	searchStrategyComboBox->addItems(dictWrapper->getSearchStrategies());

	textBrowser = new QTextBrowser(0);

	mainLayout->addWidget(entryComboBox,		0, 0);
	mainLayout->addWidget(strategiesLabel,		0, 1);
	mainLayout->addWidget(searchStrategyComboBox,	0, 2);
	mainLayout->addWidget(textBrowser,		1, 0, 3, 0);
	mainLayout->setColumnStretch(0, 1);

	setLayout(mainLayout);
	setWindowTitle("qdict");

	const int w(540);
	const int h(300);
	resize(w,h);

	// place to center
	QRect rect = QApplication::desktop()->screenGeometry();
	int x1, y1, x2, y2;
	rect.getCoords(&x1, &y1, &x2, &y2);
	move(x2/2-w/2, y2/2-h/2);

	QObject::connect(entryComboBox, SIGNAL(activated(const QString &)),
			this, SLOT(activatedCB(const QString &)));

	QObject::connect(textBrowser, SIGNAL(anchorClicked(const QUrl &)),
			this, SLOT(anchorClickedCB(const QUrl &)));
}


void MainWidget::activatedCB(const QString &text)
{
	qDebug() << "MainWidget::activatedCB";

	setWindowTitle("qdict: "+text);
	textBrowser->setOpenLinks(true);

	int strategy = searchStrategyComboBox->currentIndex();
	QString html(dictWrapper->translateToHtmlString(text,strategy));
	textBrowser->setHtml(html);

	// this is very unlikely, but maybe a link leads to an undefined word
	textBrowser->setOpenLinks(
			!(html.contains(QRegExp("No definitions found"))));
}


void MainWidget::anchorClickedCB(const QUrl &url)
{
	qDebug() << "MainWidget::anchorClickedCB";

	// open link with external browser or with QTextBrowser.
	if (textBrowser->openLinks()) {
		setWindowTitle("qdict: "+url.toString());

		QString html = dictWrapper->translateToHtmlString(
							url.toString(),0);

		// if the expr. has not been found in the DB
		textBrowser->setOpenLinks(
			!(html.contains(QRegExp("No definitions found"))));

		textBrowser->setHtml(html);

	} else {
		QDesktopServices::openUrl(url);
	}
}
