#include "UnitTest.h"

// nasty, but needed to test private members
#define private public

#include "DictWrapper.h"
#include "Gcide.h"


void testQDict::test_DictWrapper_getStdoutFromCommand()
{
	DictWrapper *dict = new DictWrapper();

	QString cmd;
	QString out;

	// simple one
	cmd = "echo hello";
	out = "hello\n";
	QCOMPARE ( dict->getStdoutFromCommand(cmd), out );

	// dict with silly word
	cmd = "dict -d gcide asdasd 2>&1";
	out = "No definitions found for \"asdasd\"\n";
	QCOMPARE ( dict->getStdoutFromCommand(cmd), out );

	// real dict test
	cmd = "dict -d gcide cat | head -n 5";
	out = 	"3 definitions found\n" \
		"\n" \
		"From The Collaborative International Dictionary of English " \
		"v.0.48 [gcide]:\n" \
		"\n" \
		"  cat \\cat\\ (k[a^]t), n. [AS. cat; " \
		"akin to D. & Dan. kat, Sw.\n";
	QCOMPARE ( dict->getStdoutFromCommand(cmd), out );

	delete dict;
}

void testQDict::test_Qcide_replaceLinks()
{
	Gcide *gcide = new Gcide();
	QStringList testLines;
	QStringList::iterator it;
	QString out;

	testLines << "line with no links";
	it = testLines.begin();
	out = "line with no links";
	gcide->replaceLinks(it);
	QCOMPARE ( *it, out );

	testLines.clear();
	testLines << "line with one {cat} link";
	it = testLines.begin();
	out = "line with one <a href=\"cat\">cat</a> link";
	gcide->replaceLinks(it);
	QCOMPARE ( *it, out );

	testLines.clear();
	testLines << "line with multiple {cat}, {dog} link";
	it = testLines.begin();
	out = "line with multiple <a href=\"cat\">cat</a>, " \
		"<a href=\"dog\">dog</a> link";
	gcide->replaceLinks(it);
	QCOMPARE ( *it, out );

	delete gcide;
}

void testQDict::test_Qcide_replaceNote()
{
	Gcide *gcide = new Gcide();
	QStringList testLines;
	QStringList::iterator it;
	QString out;

	testLines << "line with no note";
	it = testLines.begin();
	out = "line with no note";
	gcide->replaceNote(it);
	QCOMPARE ( *it, out );

	testLines.clear();
	testLines << "     Note: this is a note";
	it = testLines.begin();
	out = "<p><b>Note:</b> this is a note";
	gcide->replaceNote(it);
	QCOMPARE ( *it, out );

	testLines.clear();
	testLines << "   Note: there are less then 5 space at start";
	it = testLines.begin();
	out = "   Note: there are less then 5 space at start";
	gcide->replaceNote(it);
	QCOMPARE ( *it, out );

	testLines.clear();
	testLines << "       Note: there are more then 5 space at start";
	it = testLines.begin();
	out = "       Note: there are more then 5 space at start";
	gcide->replaceNote(it);
	QCOMPARE ( *it, out );

	delete gcide;
}


void testQDict::test_Qcide_replaceDb()
{
	Gcide *qcide = new Gcide();
	QStringList testLines;
	QStringList::iterator it;
	QString out;

	testLines << "line with no source";
	it = testLines.begin();
	out = "line with no source";
	qcide->replaceDb(it);
	QCOMPARE ( *it, out );

	testLines.clear();
	testLines << "line with a [1913 Webster] source";
	it = testLines.begin();
	out = "line with a [1913 Webster] source";
	qcide->replaceDb(it);
	QCOMPARE ( *it, out );

	testLines.clear();
	testLines << "    [1913 Webster] source";
	it = testLines.begin();
	out = "    [1913 Webster] source";
	qcide->replaceDb(it);
	QCOMPARE ( *it, out );

	testLines.clear();
	testLines << "        [1913 Webster] source";
	it = testLines.begin();
	out = "<i>[1913 Webster]</i> source";
	qcide->replaceDb(it);
	QCOMPARE ( *it, out );

	delete qcide;
}

void testQDict::test_Qcide_markQuotation()
{
	Gcide *qcide = new Gcide();
	QStringList testLines;
	QStringList::iterator it;
	QString out;

	testLines << "                 I said ...";
	it = testLines.begin();
	out = "<p><i>I said ...";
	qcide->prev_line_was_empty = true;
	qcide->markQuotation(it);
	QCOMPARE ( *it, out );


	delete qcide;
}

void testQDict::test_Qcide_markSource()
{
	Gcide *qcide = new Gcide();
	QStringList testLines;
	QStringList::iterator it;
	QString out;
	QString out2;

	qcide->in_quotation = true;
	testLines << "cat -- by me" << " ";
	it = testLines.begin();
	out = "cat </i>-- by me";
	out2 = " <b>";
	qcide->markSource(it);

	QCOMPARE ( *(it-1), out );
	QCOMPARE ( *it, out2 );

	delete qcide;
}


QTEST_MAIN(testQDict)
